@@ -46,9 +46,15 @@ class Constants:
         self.fair_threshold_high  = config.getfloat('threshold', 'fair_threshold_high')
         self.fair_neutral_threshold_low = config.getfloat('threshold', 'fair_neutral_tolerance_low')
         self.fair_neutral_threshold_high = config.getfloat('threshold', 'fair_neutral_tolerance_high')
+        self.fair_neutral_tolerance = config.getfloat('threshold', 'fair_neutral_tolerance')
         self.proportion_of_interpolation_fitting_low = config.getfloat('threshold', 'proportion_of_interpolation_fitting_low')
         self.proportion_of_interpolation_fitting_high = config.getfloat('threshold', 'proportion_of_interpolation_fitting_high')
         self.selection_threshold = config.getfloat('threshold', 'selection_threshold')
+        self.tradeoff_threshold_bins = config.getint('threshold', 'tradeoff_threshold_bins')
+        self.tradeoff_uplift_min_threshold = config.getfloat('threshold', 'tradeoff_uplift_min_threshold')
+        self.tradeoff_uplift_max_threshold = config.getfloat('threshold', 'tradeoff_uplift_max_threshold')
+        self.tradeoff_min_threshold = config.getfloat('threshold', 'tradeoff_min_threshold')
+        self.tradeoff_max_threshold = config.getfloat('threshold', 'tradeoff_max_threshold')
         self.k = config.getint('default', 'k')
         self.array_size = config.getint('default', 'array_size')
         self.decimals = config.getint('default', 'decimals')
